# rc file for the activity
# Must have one section called 'default'
# Put all the options in this section.
# Format must follow the Python ConfigParser format

[DEFAULT]

[seniorplay]

[childsplay]

[braintrainer]
