# AUTO-GENERATED MODULE, DON'T EDIT
# This module holds all the paths needed for childsplay_sp.
import os, sys

try:
    import IamBinary
except ImportError:
    print "No binary"
    BASEDIR = sys.path[0]
    os.chdir(BASEDIR)

DOCDIR = 'doc/childsplay_sp'
BASEDIR = os.getcwd()
SHARELIBDATADIR = os.path.join(BASEDIR, 'lib')
ALPHABETDIR = os.path.join(BASEDIR,'alphabetsounds')
LOCALEDIR = os.path.join(BASEDIR, 'locale')
WWWDIR = 'www/backend'
