# rc file for the activity
# Must have one section called 'default'
# Put all the options in this section.
# Format must follow the Python ConfigParser format

[DEFAULT]

# number of questions in each level
questions = 10

# retry is only used with 4 answers.
# retry 1 means one chance to pick a correct answer 
retry = 2

# autolevel_up means that the activity will suggest a level higher
# when the user are performing better than average
# Possible values 0 or 1
autolevel_up = 1

# Value indicates when the level up suggestion is done.
# In the case of the quiz activity a value of 5 means after 5 correct answers.
autolevel_value = 3

dt_questions = 1

# Put any theme specific options in the proper sections.
# Any option missing here will be used from the DEFAULT section
[seniorplay]

[braintrainer]

[childsplay]
